from typing import Optional

from frostfs_testlib.cli.cli_command import CliCommand
from frostfs_testlib.shell import CommandResult


class FrostfsCliBearer(CliCommand):
    def create(
        self,
        rpc_endpoint: str,
        out: str,
        issued_at: Optional[str] = None,
        expire_at: Optional[str] = None,
        not_valid_before: Optional[str] = None,
        ape: Optional[str] = None,
        eacl: Optional[str] = None,
        owner: Optional[str] = None,
        json: Optional[bool] = False,
        impersonate: Optional[bool] = False,
        wallet: Optional[str] = None,
        address: Optional[str] = None,
    ) -> CommandResult:
        """Create bearer token.

        All epoch flags can be specified relative to the current epoch with the +n syntax.
        In this case --rpc-endpoint flag should be specified and the epoch in bearer token
        is set to current epoch + n.
        """
        return self._execute(
            "bearer create",
            **{param: value for param, value in locals().items() if param not in ["self"]},
        )

    def generate_ape_override(
        self,
        chain_id: Optional[str] = None,
        chain_id_hex: Optional[str] = None,
        cid: Optional[str] = None,
        output: Optional[str] = None,
        path: Optional[str] = None,
        rule: Optional[str] = None,
        wallet: Optional[str] = None,
        address: Optional[str] = None,
    ) -> CommandResult:
        """Generate APE override by target and APE chains. Util command.

        Generated APE override can be dumped to a file in JSON format that is passed to
        "create" command.
        """

        return self._execute(
            "bearer generate-ape-override",
            **{param: value for param, value in locals().items() if param not in ["self"]},
        )
