from typing import Optional

from frostfs_testlib.cli.cli_command import CliCommand
from frostfs_testlib.shell import CommandResult


class FrostfsCliTree(CliCommand):
    def healthcheck(
        self,
        wallet: Optional[str] = None,
        rpc_endpoint: Optional[str] = None,
        timeout: Optional[str] = None,
    ) -> CommandResult:
        """Get internal balance of FrostFS account

        Args:
            address: Address of wallet account.
            owner: Owner of balance account (omit to use owner from private key).
            rpc_endpoint: Remote node address (as 'multiaddr' or '<host>:<port>').
            wallet: WIF (NEP-2) string or path to the wallet or binary key.

        Returns:
            Command's result.

        """
        return self._execute(
            "tree healthcheck",
            **{param: value for param, value in locals().items() if param not in ["self"]},
        )
    
    def list(
            self,
            cid: str,
            rpc_endpoint: Optional[str] = None,
            wallet: Optional[str] = None,
            timeout: Optional[str] = None,
    ) -> CommandResult:
        """Get Tree List

        Args:
            cid: Container ID.
            rpc_endpoint: Remote node address (as 'multiaddr' or '<host>:<port>').
            wallet: WIF (NEP-2) string or path to the wallet or binary key.
            timeout: duration Timeout for the operation (default 15 s)

        Returns:
            Command's result.

        """
        return self._execute(
            "tree list",
            **{param: value for param, value in locals().items() if param not in ["self"]},
        )
