import logging
from typing import Optional

from frostfs_testlib import reporter
from frostfs_testlib.cli import FrostfsCli
from frostfs_testlib.plugins import load_plugin
from frostfs_testlib.resources.cli import CLI_DEFAULT_TIMEOUT, FROSTFS_CLI_EXEC
from frostfs_testlib.shell import Shell
from frostfs_testlib.storage.dataclasses.wallet import WalletInfo

logger = logging.getLogger("NeoLogger")



@reporter.step("Get Tree List")
def get_tree_list(
    wallet: WalletInfo,
    cid: str,
    shell: Shell,
    endpoint: str,
    timeout: Optional[str] = CLI_DEFAULT_TIMEOUT,
) -> None:
    """
    A wrapper for `frostfs-cli tree list` call.
    Args:
        wallet (WalletInfo): path to a wallet on whose behalf we delete the container
        cid (str): ID of the container to delete
        shell: executor for cli command
        endpoint: FrostFS endpoint to send request to, appends to `--rpc-endpoint` key
        timeout: Timeout for the operation.
    This function doesn't return anything.
    """

    cli = FrostfsCli(shell, FROSTFS_CLI_EXEC, wallet.config_path)
    cli.tree.list(cid=cid, rpc_endpoint=endpoint, timeout=timeout)
