import logging

from frostfs_testlib import reporter
from frostfs_testlib.shell import Shell
from frostfs_testlib.steps.cli.object import head_object
from frostfs_testlib.storage.dataclasses.wallet import WalletInfo

logger = logging.getLogger("NeoLogger")


@reporter.step("Verify Head Tombstone")
def verify_head_tombstone(wallet: WalletInfo, cid: str, oid_ts: str, oid: str, shell: Shell, endpoint: str):
    header = head_object(wallet, cid, oid_ts, shell=shell, endpoint=endpoint)["header"]

    s_oid = header["sessionToken"]["body"]["object"]["target"]["objects"]
    logger.info(f"Header Session OIDs is {s_oid}")
    logger.info(f"OID is {oid}")

    assert header["containerID"] == cid, "Tombstone Header CID is wrong"
    assert header["ownerID"] == wallet.get_address_from_json(0), "Tombstone Owner ID is wrong"
    assert header["objectType"] == "TOMBSTONE", "Header Type isn't Tombstone"
    assert header["sessionToken"]["body"]["object"]["verb"] == "DELETE", "Header Session Type isn't DELETE"
    assert header["sessionToken"]["body"]["object"]["target"]["container"] == cid, "Header Session ID is wrong"
    assert oid in header["sessionToken"]["body"]["object"]["target"]["objects"], "Header Session OID is wrong"
